#ifndef _SOLVE_TRIDIAGONAL_H_
#define _SOLVE_TRIDIAGONAL_H_

/// An in-place solver of nxn linear system of equations (Ax=f) with tridiagonal sparse matrix A.
/// @param a the diagonal bellow the main diagonal with n-1 entries
/// @param c the main diagonal with n entries
/// @param b the diagonal above the main diagonal with n-1 entries
/// @param f the right hand side vector with n components
/// @param n the number of variables/equations
/// @note The solution vector x is returned in c array.
/// @note The algorithm runs with linear complexity.
/// @note The method works only for diagonally dominant matrices with nonzero diagonal entries.
template <typename T>
void solveTridiagonal(T *a, T *c, T *b, T *f, int n) {

    for (int i=1; i<n; i++) {
		T m = a[i]/c[i-1];
		c[i] -= m*b[i-1];
		f[i] -= m*f[i-1];
    }

    c[n-1] = f[n-1]/c[n-1];

    for (int i=n-2; i>=0; i--) {
		c[i] = (f[i] - b[i]*c[i+1])/c[i];
	}
}

#endif //_SOLVE_TRIDIAGONAL_H_